@@ -20,7 +20,6 @@ import struct
 import subprocess
 import sys
 import tempfile
-from datetime import datetime, timedelta, timezone
 from hashlib import pbkdf2_hmac
 from http.cookiejar import Cookie
 from . import aes, text, util
@@ -34,21 +33,21 @@ logger = logging.getLogger("cookies")
 
 
 def load_cookies(cookiejar, browser_specification):
-    browser_name, profile, keyring, container = \
+    browser_name, profile, keyring, container, domain = \
         _parse_browser_specification(*browser_specification)
     if browser_name == "firefox":
-        load_cookies_firefox(cookiejar, profile, container)
+        load_cookies_firefox(cookiejar, profile, container, domain)
     elif browser_name == "safari":
-        load_cookies_safari(cookiejar, profile)
+        load_cookies_safari(cookiejar, profile, domain)
     elif browser_name in SUPPORTED_BROWSERS_CHROMIUM:
-        load_cookies_chrome(cookiejar, browser_name, profile, keyring)
+        load_cookies_chrome(cookiejar, browser_name, profile, keyring, domain)
     else:
         raise ValueError("unknown browser '{}'".format(browser_name))
 
 
-def load_cookies_firefox(cookiejar, profile=None, container=None):
+def load_cookies_firefox(cookiejar, profile=None, container=None, domain=None):
     path, container_id = _firefox_cookies_database(profile, container)
-    with DatabaseCopy(path) as db:
+    with DatabaseConnection(path) as db:
 
         sql = ("SELECT name, value, host, path, isSecure, expiry "
                "FROM moz_cookies")
@@ -60,6 +59,13 @@ def load_cookies_firefox(cookiejar, profile=None, container=None):
             sql += " WHERE originAttributes LIKE ? OR originAttributes LIKE ?"
             uid = "%userContextId={}".format(container_id)
             parameters = (uid, uid + "&%")
+        elif domain:
+            if domain[0] == ".":
+                sql += " WHERE host == ? OR host LIKE ?"
+                parameters = (domain[1:], "%" + domain)
+            else:
+                sql += " WHERE host == ? OR host == ?"
+                parameters = (domain, "." + domain)
 
         set_cookie = cookiejar.set_cookie
         for name, value, domain, path, secure, expires in db.execute(
@@ -69,9 +75,10 @@ def load_cookies_firefox(cookiejar, profile=None, container=None):
                 domain, bool(domain), domain.startswith("."),
                 path, bool(path), secure, expires, False, None, None, {},
             ))
+        _log_info("Extracted %s cookies from Firefox", len(cookiejar))
 
 
-def load_cookies_safari(cookiejar, profile=None):
+def load_cookies_safari(cookiejar, profile=None, domain=None):
     """Ref.: https://github.com/libyal/dtformats/blob
              /main/documentation/Safari%20Cookies.asciidoc
     - This data appears to be out of date
@@ -87,27 +94,40 @@ def load_cookies_safari(cookiejar, profile=None):
         _safari_parse_cookies_page(p.read_bytes(page_size), cookiejar)
 
 
-def load_cookies_chrome(cookiejar, browser_name, profile, keyring):
+def load_cookies_chrome(cookiejar, browser_name, profile=None,
+                        keyring=None, domain=None):
     config = _get_chromium_based_browser_settings(browser_name)
     path = _chrome_cookies_database(profile, config)
-    logger.debug("Extracting cookies from %s", path)
+    _log_debug("Extracting cookies from %s", path)
 
-    with DatabaseCopy(path) as db:
+    with DatabaseConnection(path) as db:
         db.text_factory = bytes
         decryptor = get_cookie_decryptor(
-            config["directory"], config["keyring"], keyring=keyring)
+            config["directory"], config["keyring"], keyring)
+
+        if domain:
+            if domain[0] == ".":
+                condition = " WHERE host_key == ? OR host_key LIKE ?"
+                parameters = (domain[1:], "%" + domain)
+            else:
+                condition = " WHERE host_key == ? OR host_key == ?"
+                parameters = (domain, "." + domain)
+        else:
+            condition = ""
+            parameters = ()
 
         try:
             rows = db.execute(
                 "SELECT host_key, name, value, encrypted_value, path, "
-                "expires_utc, is_secure FROM cookies")
+                "expires_utc, is_secure FROM cookies" + condition, parameters)
         except sqlite3.OperationalError:
             rows = db.execute(
                 "SELECT host_key, name, value, encrypted_value, path, "
-                "expires_utc, secure FROM cookies")
+                "expires_utc, secure FROM cookies" + condition, parameters)
 
         set_cookie = cookiejar.set_cookie
-        failed_cookies = unencrypted_cookies = 0
+        failed_cookies = 0
+        unencrypted_cookies = 0
 
         for domain, name, value, enc_value, path, expires, secure in rows:
 
@@ -135,11 +155,11 @@ def load_cookies_chrome(cookiejar, browser_name, profile, keyring):
     else:
         failed_message = ""
 
-    logger.info("Extracted %s cookies from %s%s",
-                len(cookiejar), browser_name, failed_message)
-    counts = decryptor.cookie_counts.copy()
+    _log_info("Extracted %s cookies from %s%s",
+              len(cookiejar), browser_name.capitalize(), failed_message)
+    counts = decryptor.cookie_counts
     counts["unencrypted"] = unencrypted_cookies
-    logger.debug("cookie version breakdown: %s", counts)
+    _log_debug("Cookie version breakdown: %s", counts)
 
 
 # --------------------------------------------------------------------
@@ -157,11 +177,11 @@ def _firefox_cookies_database(profile=None, container=None):
     if path is None:
         raise FileNotFoundError("Unable to find Firefox cookies database in "
                                 "{}".format(search_root))
-    logger.debug("Extracting cookies from %s", path)
+    _log_debug("Extracting cookies from %s", path)
 
     if container == "none":
         container_id = False
-        logger.debug("Only loading cookies not belonging to any container")
+        _log_debug("Only loading cookies not belonging to any container")
 
     elif container:
         containers_path = os.path.join(
@@ -171,8 +191,8 @@ def _firefox_cookies_database(profile=None, container=None):
             with open(containers_path) as file:
                 identities = util.json_loads(file.read())["identities"]
         except OSError:
-            logger.error("Unable to read Firefox container database at %s",
-                         containers_path)
+            _log_error("Unable to read Firefox container database at '%s'",
+                       containers_path)
             raise
         except KeyError:
             identities = ()
@@ -183,10 +203,10 @@ def _firefox_cookies_database(profile=None, container=None):
                 container_id = context["userContextId"]
                 break
         else:
-            raise ValueError("Unable to find Firefox container {}".format(
+            raise ValueError("Unable to find Firefox container '{}'".format(
                 container))
-        logger.debug("Only loading cookies from container '%s' (ID %s)",
-                     container, container_id)
+        _log_debug("Only loading cookies from container '%s' (ID %s)",
+                   container, container_id)
     else:
         container_id = None
 
@@ -195,9 +215,11 @@ def _firefox_cookies_database(profile=None, container=None):
 
 def _firefox_browser_directory():
     if sys.platform in ("win32", "cygwin"):
-        return os.path.expandvars(r"%APPDATA%\Mozilla\Firefox\Profiles")
+        return os.path.expandvars(
+            r"%APPDATA%\Mozilla\Firefox\Profiles")
     if sys.platform == "darwin":
-        return os.path.expanduser("~/Library/Application Support/Firefox")
+        return os.path.expanduser(
+            "~/Library/Application Support/Firefox/Profiles")
     return os.path.expanduser("~/.mozilla/firefox")
 
 
@@ -209,7 +231,7 @@ def _safari_cookies_database():
         path = os.path.expanduser("~/Library/Cookies/Cookies.binarycookies")
         return open(path, "rb")
     except FileNotFoundError:
-        logger.debug("Trying secondary cookie location")
+        _log_debug("Trying secondary cookie location")
         path = os.path.expanduser("~/Library/Containers/com.apple.Safari/Data"
                                   "/Library/Cookies/Cookies.binarycookies")
         return open(path, "rb")
@@ -224,13 +246,13 @@ def _safari_parse_cookies_header(data):
     return page_sizes, p.cursor
 
 
-def _safari_parse_cookies_page(data, jar):
+def _safari_parse_cookies_page(data, cookiejar, domain=None):
     p = DataParser(data)
     p.expect_bytes(b"\x00\x00\x01\x00", "page signature")
     number_of_cookies = p.read_uint()
     record_offsets = [p.read_uint() for _ in range(number_of_cookies)]
     if number_of_cookies == 0:
-        logger.debug("a cookies page of size %s has no cookies", len(data))
+        _log_debug("Cookies page of size %s has no cookies", len(data))
         return
 
     p.skip_to(record_offsets[0], "unknown page header field")
@@ -238,12 +260,12 @@ def _safari_parse_cookies_page(data, jar):
     for i, record_offset in enumerate(record_offsets):
         p.skip_to(record_offset, "space between records")
         record_length = _safari_parse_cookies_record(
-            data[record_offset:], jar)
+            data[record_offset:], cookiejar, domain)
         p.read_bytes(record_length)
     p.skip_to_end("space in between pages")
 
 
-def _safari_parse_cookies_record(data, cookiejar):
+def _safari_parse_cookies_record(data, cookiejar, host=None):
     p = DataParser(data)
     record_size = p.read_uint()
     p.skip(4, "unknown record field 1")
@@ -262,6 +284,14 @@ def _safari_parse_cookies_record(data, cookiejar):
         p.skip_to(domain_offset)
         domain = p.read_cstring()
 
+        if host:
+            if host[0] == ".":
+                if host[1:] != domain and not domain.endswith(host):
+                    return record_size
+            else:
+                if host != domain and ("." + host) != domain:
+                    return record_size
+
         p.skip_to(name_offset)
         name = p.read_cstring()
 
@@ -271,8 +301,7 @@ def _safari_parse_cookies_record(data, cookiejar):
         p.skip_to(value_offset)
         value = p.read_cstring()
     except UnicodeDecodeError:
-        logger.warning("failed to parse Safari cookie "
-                       "because UTF-8 decoding failed")
+        _log_warning("Failed to parse Safari cookie")
         return record_size
 
     p.skip_to(record_size, "space at the end of the record")
@@ -300,7 +329,7 @@ def _chrome_cookies_database(profile, config):
     elif config["profiles"]:
         search_root = os.path.join(config["directory"], profile)
     else:
-        logger.warning("%s does not support profiles", config["browser"])
+        _log_warning("%s does not support profiles", config["browser"])
         search_root = config["directory"]
 
     path = _find_most_recently_used_file(search_root, "Cookies")
@@ -412,18 +441,17 @@ class ChromeCookieDecryptor:
         raise NotImplementedError("Must be implemented by sub classes")
 
 
-def get_cookie_decryptor(browser_root, browser_keyring_name, *, keyring=None):
+def get_cookie_decryptor(browser_root, browser_keyring_name, keyring=None):
     if sys.platform in ("win32", "cygwin"):
         return WindowsChromeCookieDecryptor(browser_root)
     elif sys.platform == "darwin":
         return MacChromeCookieDecryptor(browser_keyring_name)
     else:
-        return LinuxChromeCookieDecryptor(
-            browser_keyring_name, keyring=keyring)
+        return LinuxChromeCookieDecryptor(browser_keyring_name, keyring)
 
 
 class LinuxChromeCookieDecryptor(ChromeCookieDecryptor):
-    def __init__(self, browser_keyring_name, *, keyring=None):
+    def __init__(self, browser_keyring_name, keyring=None):
         self._v10_key = self.derive_key(b"peanuts")
         password = _get_linux_keyring_password(browser_keyring_name, keyring)
         self._v11_key = None if password is None else self.derive_key(password)
@@ -452,7 +480,7 @@ class LinuxChromeCookieDecryptor(ChromeCookieDecryptor):
         elif version == b"v11":
             self._cookie_counts["v11"] += 1
             if self._v11_key is None:
-                logger.warning("cannot decrypt v11 cookies: no key found")
+                _log_warning("Unable to decrypt v11 cookies: no key found")
                 return None
             return _decrypt_aes_cbc(ciphertext, self._v11_key)
 
@@ -486,7 +514,7 @@ class MacChromeCookieDecryptor(ChromeCookieDecryptor):
         if version == b"v10":
             self._cookie_counts["v10"] += 1
             if self._v10_key is None:
-                logger.warning("cannot decrypt v10 cookies: no key found")
+                _log_warning("Unable to decrypt v10 cookies: no key found")
                 return None
 
             return _decrypt_aes_cbc(ciphertext, self._v10_key)
@@ -516,7 +544,7 @@ class WindowsChromeCookieDecryptor(ChromeCookieDecryptor):
         if version == b"v10":
             self._cookie_counts["v10"] += 1
             if self._v10_key is None:
-                logger.warning("cannot decrypt v10 cookies: no key found")
+                _log_warning("Unable to decrypt v10 cookies: no key found")
                 return None
 
             # https://chromium.googlesource.com/chromium/src/+/refs/heads
@@ -554,7 +582,7 @@ def _choose_linux_keyring():
     SelectBackend
     """
     desktop_environment = _get_linux_desktop_environment(os.environ)
-    logger.debug("Detected desktop environment: %s", desktop_environment)
+    _log_debug("Detected desktop environment: %s", desktop_environment)
     if desktop_environment == DE_KDE:
         return KEYRING_KWALLET
     if desktop_environment == DE_OTHER:
@@ -582,23 +610,23 @@ def _get_kwallet_network_wallet():
         )
 
         if proc.returncode != 0:
-            logger.warning("failed to read NetworkWallet")
+            _log_warning("Failed to read NetworkWallet")
             return default_wallet
         else:
             network_wallet = stdout.decode().strip()
-            logger.debug("NetworkWallet = '%s'", network_wallet)
+            _log_debug("NetworkWallet = '%s'", network_wallet)
             return network_wallet
     except Exception as exc:
-        logger.warning("exception while obtaining NetworkWallet (%s: %s)",
-                       exc.__class__.__name__, exc)
+        _log_warning("Error while obtaining NetworkWallet (%s: %s)",
+                     exc.__class__.__name__, exc)
         return default_wallet
 
 
 def _get_kwallet_password(browser_keyring_name):
-    logger.debug("using kwallet-query to obtain password from kwallet")
+    _log_debug("Using kwallet-query to obtain password from kwallet")
 
     if shutil.which("kwallet-query") is None:
-        logger.error(
+        _log_error(
             "kwallet-query command not found. KWallet and kwallet-query "
             "must be installed to read from KWallet. kwallet-query should be "
             "included in the kwallet package for your distribution")
@@ -615,14 +643,14 @@ def _get_kwallet_password(browser_keyring_name):
         )
 
         if proc.returncode != 0:
-            logger.error("kwallet-query failed with return code {}. "
-                         "Please consult the kwallet-query man page "
-                         "for details".format(proc.returncode))
+            _log_error("kwallet-query failed with return code {}. "
+                       "Please consult the kwallet-query man page "
+                       "for details".format(proc.returncode))
             return b""
 
         if stdout.lower().startswith(b"failed to read"):
-            logger.debug("Failed to read password from kwallet. "
-                         "Using empty string instead")
+            _log_debug("Failed to read password from kwallet. "
+                       "Using empty string instead")
             # This sometimes occurs in KDE because chrome does not check
             # hasEntry and instead just tries to read the value (which
             # kwallet returns "") whereas kwallet-query checks hasEntry.
@@ -633,13 +661,12 @@ def _get_kwallet_password(browser_keyring_name):
             # random password and store it, but that doesn't matter here.
             return b""
         else:
-            logger.debug("password found")
             if stdout[-1:] == b"\n":
                 stdout = stdout[:-1]
             return stdout
     except Exception as exc:
-        logger.warning("exception running kwallet-query (%s: %s)",
-                       exc.__class__.__name__, exc)
+        _log_warning("Error when running kwallet-query (%s: %s)",
+                     exc.__class__.__name__, exc)
         return b""
 
 
@@ -647,7 +674,7 @@ def _get_gnome_keyring_password(browser_keyring_name):
     try:
         import secretstorage
     except ImportError:
-        logger.error("secretstorage not available")
+        _log_error("'secretstorage' Python package not available")
         return b""
 
     # Gnome keyring does not seem to organise keys in the same way as KWallet,
@@ -662,7 +689,7 @@ def _get_gnome_keyring_password(browser_keyring_name):
             if item.get_label() == label:
                 return item.get_secret()
         else:
-            logger.error("failed to read from keyring")
+            _log_error("Failed to read from GNOME keyring")
             return b""
 
 
@@ -676,7 +703,7 @@ def _get_linux_keyring_password(browser_keyring_name, keyring):
 
     if not keyring:
         keyring = _choose_linux_keyring()
-    logger.debug("Chosen keyring: %s", keyring)
+    _log_debug("Chosen keyring: %s", keyring)
 
     if keyring == KEYRING_KWALLET:
         return _get_kwallet_password(browser_keyring_name)
@@ -690,8 +717,8 @@ def _get_linux_keyring_password(browser_keyring_name, keyring):
 
 
 def _get_mac_keyring_password(browser_keyring_name):
-    logger.debug("using find-generic-password to obtain "
-                 "password from OSX keychain")
+    _log_debug("Using find-generic-password to obtain "
+               "password from OSX keychain")
     try:
         proc, stdout = Popen_communicate(
             "security", "find-generic-password",
@@ -704,28 +731,28 @@ def _get_mac_keyring_password(browser_keyring_name):
             stdout = stdout[:-1]
         return stdout
     except Exception as exc:
-        logger.warning("exception running find-generic-password (%s: %s)",
-                       exc.__class__.__name__, exc)
+        _log_warning("Error when using find-generic-password (%s: %s)",
+                     exc.__class__.__name__, exc)
         return None
 
 
 def _get_windows_v10_key(browser_root):
     path = _find_most_recently_used_file(browser_root, "Local State")
     if path is None:
-        logger.error("could not find local state file")
+        _log_error("Unable to find Local State file")
         return None
-    logger.debug("Found local state file at '%s'", path)
+    _log_debug("Found Local State file at '%s'", path)
     with open(path, encoding="utf-8") as file:
         data = util.json_loads(file.read())
     try:
         base64_key = data["os_crypt"]["encrypted_key"]
     except KeyError:
-        logger.error("no encrypted key in Local State")
+        _log_error("Unable to find encrypted key in Local State")
         return None
     encrypted_key = binascii.a2b_base64(base64_key)
     prefix = b"DPAPI"
     if not encrypted_key.startswith(prefix):
-        logger.error("invalid key")
+        _log_error("Invalid Local State key")
         return None
     return _decrypt_windows_dpapi(encrypted_key[len(prefix):])
 
@@ -777,10 +804,10 @@ class DataParser:
 
     def skip(self, num_bytes, description="unknown"):
         if num_bytes > 0:
-            logger.debug("skipping {} bytes ({}): {!r}".format(
+            _log_debug("Skipping {} bytes ({}): {!r}".format(
                 num_bytes, description, self.read_bytes(num_bytes)))
         elif num_bytes < 0:
-            raise ParserError("invalid skip of {} bytes".format(num_bytes))
+            raise ParserError("Invalid skip of {} bytes".format(num_bytes))
 
     def skip_to(self, offset, description="unknown"):
         self.skip(offset - self.cursor, description)
@@ -789,7 +816,7 @@ class DataParser:
         self.skip_to(len(self._data), description)
 
 
-class DatabaseCopy():
+class DatabaseConnection():
 
     def __init__(self, path):
         self.path = path
@@ -797,13 +824,27 @@ class DatabaseCopy():
         self.directory = None
 
     def __enter__(self):
+        try:
+            # https://www.sqlite.org/uri.html#the_uri_path
+            path = self.path.replace("?", "%3f").replace("#", "%23")
+            if util.WINDOWS:
+                path = "/" + os.path.abspath(path)
+
+            uri = "file:{}?mode=ro&immutable=1".format(path)
+            self.database = sqlite3.connect(
+                uri, uri=True, isolation_level=None, check_same_thread=False)
+            return self.database
+        except Exception as exc:
+            _log_debug("Falling back to temporary database copy (%s: %s)",
+                       exc.__class__.__name__, exc)
+
         try:
             self.directory = tempfile.TemporaryDirectory(prefix="gallery-dl-")
             path_copy = os.path.join(self.directory.name, "copy.sqlite")
             shutil.copyfile(self.path, path_copy)
-            self.database = db = sqlite3.connect(
+            self.database = sqlite3.connect(
                 path_copy, isolation_level=None, check_same_thread=False)
-            return db
+            return self.database
         except BaseException:
             if self.directory:
                 self.directory.cleanup()
@@ -811,7 +852,8 @@ class DatabaseCopy():
 
     def __exit__(self, exc, value, tb):
         self.database.close()
-        self.directory.cleanup()
+        if self.directory:
+            self.directory.cleanup()
 
 
 def Popen_communicate(*args):
@@ -893,8 +935,8 @@ def _get_linux_desktop_environment(env):
 
 
 def _mac_absolute_time_to_posix(timestamp):
-    return int((datetime(2001, 1, 1, 0, 0, tzinfo=timezone.utc) +
-                timedelta(seconds=timestamp)).timestamp())
+    # 978307200 is timestamp of 2001-01-01 00:00:00
+    return 978307200 + int(timestamp)
 
 
 def pbkdf2_sha1(password, salt, iterations, key_length):
@@ -902,31 +944,25 @@ def pbkdf2_sha1(password, salt, iterations, key_length):
 
 
 def _decrypt_aes_cbc(ciphertext, key, initialization_vector=b" " * 16):
-    plaintext = aes.unpad_pkcs7(
-        aes.aes_cbc_decrypt_bytes(ciphertext, key, initialization_vector))
     try:
-        return plaintext.decode()
+        return aes.unpad_pkcs7(aes.aes_cbc_decrypt_bytes(
+            ciphertext, key, initialization_vector)).decode()
     except UnicodeDecodeError:
-        logger.warning("failed to decrypt cookie (AES-CBC) because UTF-8 "
-                       "decoding failed. Possibly the key is wrong?")
-        return None
+        _log_warning("Failed to decrypt cookie (AES-CBC Unicode)")
+    except ValueError:
+        _log_warning("Failed to decrypt cookie (AES-CBC)")
+    return None
 
 
 def _decrypt_aes_gcm(ciphertext, key, nonce, authentication_tag):
     try:
-        plaintext = aes.aes_gcm_decrypt_and_verify_bytes(
-            ciphertext, key, authentication_tag, nonce)
-    except ValueError:
-        logger.warning("failed to decrypt cookie (AES-GCM) because MAC check "
-                       "failed. Possibly the key is wrong?")
-        return None
-
-    try:
-        return plaintext.decode()
+        return aes.aes_gcm_decrypt_and_verify_bytes(
+            ciphertext, key, authentication_tag, nonce).decode()
     except UnicodeDecodeError:
-        logger.warning("failed to decrypt cookie (AES-GCM) because UTF-8 "
-                       "decoding failed. Possibly the key is wrong?")
-        return None
+        _log_warning("Failed to decrypt cookie (AES-GCM Unicode)")
+    except ValueError:
+        _log_warning("Failed to decrypt cookie (AES-GCM MAC)")
+    return None
 
 
 def _decrypt_windows_dpapi(ciphertext):
@@ -954,7 +990,7 @@ def _decrypt_windows_dpapi(ciphertext):
         ctypes.byref(blob_out)  # pDataOut
     )
     if not ret:
-        logger.warning("failed to decrypt with DPAPI")
+        _log_warning("Failed to decrypt cookie (DPAPI)")
         return None
 
     result = ctypes.string_at(blob_out.pbData, blob_out.cbData)
@@ -979,12 +1015,29 @@ def _is_path(value):
 
 
 def _parse_browser_specification(
-        browser, profile=None, keyring=None, container=None):
+        browser, profile=None, keyring=None, container=None, domain=None):
     browser = browser.lower()
     if browser not in SUPPORTED_BROWSERS:
-        raise ValueError("unsupported browser '{}'".format(browser))
+        raise ValueError("Unsupported browser '{}'".format(browser))
     if keyring and keyring not in SUPPORTED_KEYRINGS:
-        raise ValueError("unsupported keyring '{}'".format(keyring))
+        raise ValueError("Unsupported keyring '{}'".format(keyring))
     if profile and _is_path(profile):
         profile = os.path.expanduser(profile)
-    return browser, profile, keyring, container
+    return browser, profile, keyring, container, domain
+
+
+_log_cache = set()
+_log_debug = logger.debug
+_log_info = logger.info
+
+
+def _log_warning(msg, *args):
+    if msg not in _log_cache:
+        _log_cache.add(msg)
+        logger.warning(msg, *args)
+
+
+def _log_error(msg, *args):
+    if msg not in _log_cache:
+        _log_cache.add(msg)
+        logger.error(msg, *args)
