@@ -13,12 +13,7 @@ import mimetypes
 from requests.exceptions import RequestException, ConnectionError, Timeout
 from .common import DownloaderBase
 from .. import text, util
-
 from ssl import SSLError
-try:
-    from OpenSSL.SSL import Error as OpenSSLError
-except ImportError:
-    OpenSSLError = SSLError
 
 
 class HttpDownloader(DownloaderBase):
@@ -200,13 +195,15 @@ class HttpDownloader(DownloaderBase):
                     self.log.warning(
                         "File size smaller than allowed minimum (%s < %s)",
                         size, self.minsize)
-                    return False
+                    pathfmt.temppath = ""
+                    return True
                 if self.maxsize and size > self.maxsize:
                     self.release_conn(response)
                     self.log.warning(
                         "File size larger than allowed maximum (%s > %s)",
                         size, self.maxsize)
-                    return False
+                    pathfmt.temppath = ""
+                    return True
 
             build_path = False
 
@@ -247,7 +244,7 @@ class HttpDownloader(DownloaderBase):
                     file_header = next(
                         content if response.raw.chunked
                         else response.iter_content(16), b"")
-                except (RequestException, SSLError, OpenSSLError) as exc:
+                except (RequestException, SSLError) as exc:
                     msg = str(exc)
                     print()
                     continue
@@ -281,7 +278,7 @@ class HttpDownloader(DownloaderBase):
                 self.out.start(pathfmt.path)
                 try:
                     self.receive(fp, content, size, offset)
-                except (RequestException, SSLError, OpenSSLError) as exc:
+                except (RequestException, SSLError) as exc:
                     msg = str(exc)
                     print()
                     continue
@@ -308,7 +305,7 @@ class HttpDownloader(DownloaderBase):
         try:
             for _ in response.iter_content(self.chunk_size):
                 pass
-        except (RequestException, SSLError, OpenSSLError) as exc:
+        except (RequestException, SSLError) as exc:
             print()
             self.log.debug(
                 "Unable to consume response body (%s: %s); "
@@ -402,6 +399,7 @@ MIME_TYPES = {
     "video/webm": "webm",
     "video/ogg" : "ogg",
     "video/mp4" : "mp4",
+    "video/quicktime": "mov",
 
     "audio/wav"  : "wav",
     "audio/x-wav": "wav",
@@ -444,6 +442,7 @@ SIGNATURE_CHECKS = {
     "psd" : lambda s: s[0:4] == b"8BPS",
     "mp4" : lambda s: (s[4:8] == b"ftyp" and s[8:11] in (
                        b"mp4", b"avc", b"iso", b"M4V")),
+    "mov" : lambda s: s[4:12] == b"ftypqt  ",
     "webm": lambda s: s[0:4] == b"\x1A\x45\xDF\xA3",
     "ogg" : lambda s: s[0:4] == b"OggS",
     "wav" : lambda s: (s[0:4] == b"RIFF" and
