@@ -10,6 +10,7 @@ import sys
 import re
 
 modules = [
+    "2ch",
     "2chan",
     "2chen",
     "35photo",
@@ -28,6 +29,7 @@ modules = [
     "bbc",
     "behance",
     "blogger",
+    "bluesky",
     "bunkr",
     "catbox",
     "chevereto",
@@ -53,7 +55,7 @@ modules = [
     "gelbooru_v01",
     "gelbooru_v02",
     "gofile",
-    "hbrowse",
+    "hatenablog",
     "hentai2read",
     "hentaicosplays",
     "hentaifoundry",
@@ -145,6 +147,7 @@ modules = [
     "smugmug",
     "soundgasm",
     "speakerdeck",
+    "steamgriddb",
     "subscribestar",
     "szurubooru",
     "tapas",
@@ -175,6 +178,7 @@ modules = [
     "weibo",
     "wikiart",
     "wikifeet",
+    "wikimedia",
     "xhamster",
     "xvideos",
     "zerochan",
@@ -190,7 +194,6 @@ modules = [
     "directlink",
     "recursive",
     "oauth",
-    "test",
     "ytdl",
     "generic",
 ]
