@@ -23,7 +23,7 @@ class GelbooruBase():
     root = "https://gelbooru.com"
     offset = 0
 
-    def _api_request(self, params, key="post"):
+    def _api_request(self, params, key="post", log=False):
         if "s" not in params:
             params["s"] = "post"
         params["api_key"] = self.api_key
@@ -32,10 +32,17 @@ class GelbooruBase():
         url = self.root + "/index.php?page=dapi&q=index&json=1"
         data = self.request(url, params=params).json()
 
-        if key not in data:
-            return ()
+        if not key:
+            return data
+
+        try:
+            posts = data[key]
+        except KeyError:
+            if log:
+                self.log.error("Incomplete API response (missing '%s')", key)
+                self.log.debug("%s", data)
+            return []
 
-        posts = data[key]
         if not isinstance(posts, list):
             return (posts,)
         return posts
@@ -114,7 +121,7 @@ class GelbooruBase():
 class GelbooruTagExtractor(GelbooruBase,
                            gelbooru_v02.GelbooruV02TagExtractor):
     """Extractor for images from gelbooru.com based on search-tags"""
-    pattern = BASE_PATTERN + r"page=post&s=list&tags=([^&#]+)"
+    pattern = BASE_PATTERN + r"page=post&s=list&tags=([^&#]*)"
     example = "https://gelbooru.com/index.php?page=post&s=list&tags=TAG"
 
 
@@ -163,34 +170,84 @@ class GelbooruFavoriteExtractor(GelbooruBase,
         params = {
             "s"    : "favorite",
             "id"   : self.favorite_id,
-            "limit": "1",
+            "limit": "2",
         }
-        count = self._api_request(params, "@attributes")[0]["count"]
+        data = self._api_request(params, None, True)
 
-        if count <= self.offset:
-            return
-        pnum, last = divmod(count + 1, self.per_page)
+        count = data["@attributes"]["count"]
+        self.log.debug("API reports %s favorite entries", count)
 
-        if self.offset >= last:
-            self.offset -= last
-            diff, self.offset = divmod(self.offset, self.per_page)
-            pnum -= diff + 1
-        skip = self.offset
+        favs = data["favorite"]
+        try:
+            order = 1 if favs[0]["id"] < favs[1]["id"] else -1
+        except LookupError as exc:
+            self.log.debug(
+                "Error when determining API favorite order (%s: %s)",
+                exc.__class__.__name__, exc)
+            order = -1
+        else:
+            self.log.debug("API yields favorites in %sscending order",
+                           "a" if order > 0 else "de")
+
+        order_favs = self.config("order-posts")
+        if order_favs and order_favs[0] in ("r", "a"):
+            self.log.debug("Returning them in reverse")
+            order = -order
+
+        if order < 0:
+            return self._pagination(params, count)
+        return self._pagination_reverse(params, count)
+
+    def _pagination(self, params, count):
+        if self.offset:
+            pnum, skip = divmod(self.offset, self.per_page)
+        else:
+            pnum = skip = 0
 
-        # paginate over them in reverse
         params["pid"] = pnum
         params["limit"] = self.per_page
 
         while True:
             favs = self._api_request(params, "favorite")
 
-            favs.reverse()
+            if not favs:
+                return
+
             if skip:
                 favs = favs[skip:]
                 skip = 0
 
             for fav in favs:
-                yield from self._api_request({"id": fav["favorite"]})
+                for post in self._api_request({"id": fav["favorite"]}):
+                    post["date_favorited"] = text.parse_timestamp(fav["added"])
+                    yield post
+
+            params["pid"] += 1
+
+    def _pagination_reverse(self, params, count):
+        pnum, last = divmod(count-1, self.per_page)
+        if self.offset > last:
+            # page number change
+            self.offset -= last
+            diff, self.offset = divmod(self.offset-1, self.per_page)
+            pnum -= diff + 1
+        skip = self.offset
+
+        params["pid"] = pnum
+        params["limit"] = self.per_page
+
+        while True:
+            favs = self._api_request(params, "favorite")
+            favs.reverse()
+
+            if skip:
+                favs = favs[skip:]
+                skip = 0
+
+            for fav in favs:
+                for post in self._api_request({"id": fav["favorite"]}):
+                    post["date_favorited"] = text.parse_timestamp(fav["added"])
+                    yield post
 
             params["pid"] -= 1
             if params["pid"] < 0:
