@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2022 Mike Fährmann
+# Copyright 2015-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -20,7 +20,7 @@ class HentaifoundryExtractor(Extractor):
     directory_fmt = ("{category}", "{user}")
     filename_fmt = "{category}_{index}_{title}.{extension}"
     archive_fmt = "{index}"
-    cookiedomain = "www.hentai-foundry.com"
+    cookies_domain = "www.hentai-foundry.com"
     root = "https://www.hentai-foundry.com"
     per_page = 25
 
@@ -72,13 +72,11 @@ class HentaifoundryExtractor(Extractor):
         extr = text.extract_from(page, page.index('id="picBox"'))
 
         data = {
+            "index"      : text.parse_int(path.rsplit("/", 2)[1]),
             "title"      : text.unescape(extr('class="imageTitle">', '<')),
             "artist"     : text.unescape(extr('/profile">', '<')),
-            "width"      : text.parse_int(extr('width="', '"')),
-            "height"     : text.parse_int(extr('height="', '"')),
-            "index"      : text.parse_int(path.rsplit("/", 2)[1]),
-            "src"        : text.urljoin(self.root, text.unescape(extr(
-                'src="', '"'))),
+            "_body"      : extr(
+                '<div class="boxbody"', '<div class="boxfooter"'),
             "description": text.unescape(text.remove_html(extr(
                 '>Description</div>', '</section>')
                 .replace("\r\n", "\n"), "", "")),
@@ -92,6 +90,20 @@ class HentaifoundryExtractor(Extractor):
                 ">Tags </span>", "</div>")),
         }
 
+        body = data["_body"]
+        if "<object " in body:
+            data["src"] = text.urljoin(self.root, text.unescape(text.extr(
+                body, 'name="movie" value="', '"')))
+            data["width"] = text.parse_int(text.extr(
+                body, "name='width' value='", "'"))
+            data["height"] = text.parse_int(text.extr(
+                body, "name='height' value='", "'"))
+        else:
+            data["src"] = text.urljoin(self.root, text.unescape(text.extr(
+                body, 'src="', '"')))
+            data["width"] = text.parse_int(text.extr(body, 'width="', '"'))
+            data["height"] = text.parse_int(text.extr(body, 'height="', '"'))
+
         return text.nameext_from_url(data["src"], data)
 
     def _parse_story(self, html):
@@ -121,13 +133,32 @@ class HentaifoundryExtractor(Extractor):
 
         return text.nameext_from_url(data["src"], data)
 
-    def _init_site_filters(self):
+    def _request_check(self, url, **kwargs):
+        self.request = self._request_original
+
+        # check for Enter button / front page
+        # and update PHPSESSID and content filters if necessary
+        response = self.request(url, **kwargs)
+        content = response.content
+        if len(content) < 5000 and \
+                b'<div id="entryButtonContainer"' in content:
+            self._init_site_filters(False)
+            response = self.request(url, **kwargs)
+        return response
+
+    def _init_site_filters(self, check_cookies=True):
         """Set site-internal filters to show all images"""
+        if check_cookies and self.cookies.get(
+                "PHPSESSID", domain=self.cookies_domain):
+            self._request_original = self.request
+            self.request = self._request_check
+            return
+
         url = self.root + "/?enterAgree=1"
         self.request(url, method="HEAD")
 
-        csrf_token = self.session.cookies.get(
-            "YII_CSRF_TOKEN", domain=self.cookiedomain)
+        csrf_token = self.cookies.get(
+            "YII_CSRF_TOKEN", domain=self.cookies_domain)
         if not csrf_token:
             self.log.warning("Unable to update site content filters")
             return
@@ -153,7 +184,6 @@ class HentaifoundryExtractor(Extractor):
             "rating_scat"     : "1",
             "rating_incest"   : "1",
             "rating_rape"     : "1",
-            "filter_media"    : "A",
             "filter_order"    : "date_new",
             "filter_type"     : "0",
             "YII_CSRF_TOKEN"  : text.unquote(text.extr(
@@ -166,7 +196,10 @@ class HentaifoundryUserExtractor(HentaifoundryExtractor):
     """Extractor for a hentaifoundry user profile"""
     subcategory = "user"
     pattern = BASE_PATTERN + r"/user/([^/?#]+)/profile"
-    test = ("https://www.hentai-foundry.com/user/Tenpura/profile",)
+    example = "https://www.hentai-foundry.com/user/USER/profile"
+
+    def initialize(self):
+        pass
 
     def items(self):
         root = self.root
@@ -187,12 +220,7 @@ class HentaifoundryPicturesExtractor(HentaifoundryExtractor):
     """Extractor for all pictures of a hentaifoundry user"""
     subcategory = "pictures"
     pattern = BASE_PATTERN + r"/pictures/user/([^/?#]+)(?:/page/(\d+))?/?$"
-    test = (
-        ("https://www.hentai-foundry.com/pictures/user/Tenpura", {
-            "url": "ebbc981a85073745e3ca64a0f2ab31fab967fc28",
-        }),
-        ("https://www.hentai-foundry.com/pictures/user/Tenpura/page/3"),
-    )
+    example = "https://www.hentai-foundry.com/pictures/user/USER"
 
     def __init__(self, match):
         HentaifoundryExtractor.__init__(self, match)
@@ -204,13 +232,7 @@ class HentaifoundryScrapsExtractor(HentaifoundryExtractor):
     subcategory = "scraps"
     directory_fmt = ("{category}", "{user}", "Scraps")
     pattern = BASE_PATTERN + r"/pictures/user/([^/?#]+)/scraps"
-    test = (
-        ("https://www.hentai-foundry.com/pictures/user/Evulchibi/scraps", {
-            "url": "7cd9c6ec6258c4ab8c44991f7731be82337492a7",
-        }),
-        ("https://www.hentai-foundry.com"
-         "/pictures/user/Evulchibi/scraps/page/3"),
-    )
+    example = "https://www.hentai-foundry.com/pictures/user/USER/scraps"
 
     def __init__(self, match):
         HentaifoundryExtractor.__init__(self, match)
@@ -224,13 +246,7 @@ class HentaifoundryFavoriteExtractor(HentaifoundryExtractor):
     directory_fmt = ("{category}", "{user}", "Favorites")
     archive_fmt = "f_{user}_{index}"
     pattern = BASE_PATTERN + r"/user/([^/?#]+)/faves/pictures"
-    test = (
-        ("https://www.hentai-foundry.com/user/Tenpura/faves/pictures", {
-            "url": "56f9ae2e89fe855e9fe1da9b81e5ec6212b0320b",
-        }),
-        ("https://www.hentai-foundry.com"
-         "/user/Tenpura/faves/pictures/page/3"),
-    )
+    example = "https://www.hentai-foundry.com/user/USER/faves/pictures"
 
     def __init__(self, match):
         HentaifoundryExtractor.__init__(self, match)
@@ -244,10 +260,7 @@ class HentaifoundryRecentExtractor(HentaifoundryExtractor):
     directory_fmt = ("{category}", "Recent Pictures", "{date}")
     archive_fmt = "r_{index}"
     pattern = BASE_PATTERN + r"/pictures/recent/(\d\d\d\d-\d\d-\d\d)"
-    test = ("https://www.hentai-foundry.com/pictures/recent/2018-09-20", {
-        "pattern": r"https://pictures.hentai-foundry.com/[^/]/[^/?#]+/\d+/",
-        "range": "20-30",
-    })
+    example = "https://www.hentai-foundry.com/pictures/recent/1970-01-01"
 
     def __init__(self, match):
         HentaifoundryExtractor.__init__(self, match)
@@ -263,10 +276,7 @@ class HentaifoundryPopularExtractor(HentaifoundryExtractor):
     directory_fmt = ("{category}", "Popular Pictures")
     archive_fmt = "p_{index}"
     pattern = BASE_PATTERN + r"/pictures/popular()"
-    test = ("https://www.hentai-foundry.com/pictures/popular", {
-        "pattern": r"https://pictures.hentai-foundry.com/[^/]/[^/?#]+/\d+/",
-        "range": "20-30",
-    })
+    example = "https://www.hentai-foundry.com/pictures/popular"
 
     def __init__(self, match):
         HentaifoundryExtractor.__init__(self, match)
@@ -278,34 +288,8 @@ class HentaifoundryImageExtractor(HentaifoundryExtractor):
     subcategory = "image"
     pattern = (r"(https?://)?(?:www\.|pictures\.)?hentai-foundry\.com"
                r"/(?:pictures/user|[^/?#])/([^/?#]+)/(\d+)")
-    test = (
-        (("https://www.hentai-foundry.com"
-          "/pictures/user/Tenpura/407501/shimakaze"), {
-            "url": "fbf2fd74906738094e2575d2728e8dc3de18a8a3",
-            "content": "91bf01497c39254b6dfb234a18e8f01629c77fd1",
-            "keyword": {
-                "artist" : "Tenpura",
-                "date"   : "dt:2016-02-22 14:41:19",
-                "description": "Thank you!",
-                "height" : 700,
-                "index"  : 407501,
-                "media"  : "Other digital art",
-                "ratings": ["Sexual content", "Contains female nudity"],
-                "score"  : int,
-                "tags"   : ["collection", "kancolle", "kantai", "shimakaze"],
-                "title"  : "shimakaze",
-                "user"   : "Tenpura",
-                "views"  : int,
-                "width"  : 495,
-            },
-        }),
-        ("http://www.hentai-foundry.com/pictures/user/Tenpura/407501/", {
-            "pattern": "http://pictures.hentai-foundry.com/t/Tenpura/407501/",
-        }),
-        ("https://www.hentai-foundry.com/pictures/user/Tenpura/407501/"),
-        ("https://pictures.hentai-foundry.com"
-         "/t/Tenpura/407501/Tenpura-407501-shimakaze.png"),
-    )
+    example = "https://www.hentai-foundry.com/pictures/user/USER/12345/TITLE"
+
     skip = Extractor.skip
 
     def __init__(self, match):
@@ -326,24 +310,7 @@ class HentaifoundryStoriesExtractor(HentaifoundryExtractor):
     subcategory = "stories"
     archive_fmt = "s_{index}"
     pattern = BASE_PATTERN + r"/stories/user/([^/?#]+)(?:/page/(\d+))?/?$"
-    test = ("https://www.hentai-foundry.com/stories/user/SnowWolf35", {
-        "count": ">= 35",
-        "keyword": {
-            "author"     : "SnowWolf35",
-            "chapters"   : int,
-            "comments"   : int,
-            "date"       : "type:datetime",
-            "description": str,
-            "index"      : int,
-            "rating"     : int,
-            "ratings"    : list,
-            "status"     : "re:(Inc|C)omplete",
-            "title"      : str,
-            "user"       : "SnowWolf35",
-            "views"      : int,
-            "words"      : int,
-        },
-    })
+    example = "https://www.hentai-foundry.com/stories/user/USER"
 
     def items(self):
         self._init_site_filters()
@@ -362,11 +329,8 @@ class HentaifoundryStoryExtractor(HentaifoundryExtractor):
     subcategory = "story"
     archive_fmt = "s_{index}"
     pattern = BASE_PATTERN + r"/stories/user/([^/?#]+)/(\d+)"
-    test = (("https://www.hentai-foundry.com/stories/user/SnowWolf35"
-             "/26416/Overwatch-High-Chapter-Voting-Location"), {
-        "url": "5a67cfa8c3bf7634c8af8485dd07c1ea74ee0ae8",
-        "keyword": {"title": "Overwatch High Chapter Voting Location"},
-    })
+    example = "https://www.hentai-foundry.com/stories/user/USER/12345/TITLE"
+
     skip = Extractor.skip
 
     def __init__(self, match):
