@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2017-2022 Mike Fährmann
+# Copyright 2017-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -20,20 +20,12 @@ class MangafoxChapterExtractor(ChapterExtractor):
     root = "https://m.fanfox.net"
     pattern = BASE_PATTERN + \
         r"(/manga/[^/?#]+/((?:v([^/?#]+)/)?c(\d+)([^/?#]*)))"
-    test = (
-        ("http://fanfox.net/manga/kidou_keisatsu_patlabor/v05/c006.2/1.html", {
-            "keyword": "5661dab258d42d09d98f194f7172fb9851a49766",
-            "content": "5c50c252dcf12ffecf68801f4db8a2167265f66c",
-        }),
-        ("http://mangafox.me/manga/kidou_keisatsu_patlabor/v05/c006.2/"),
-        ("http://fanfox.net/manga/black_clover/vTBD/c295/1.html"),
-    )
+    example = "https://fanfox.net/manga/TITLE/v01/c001/1.html"
 
     def __init__(self, match):
         base, self.cstr, self.volume, self.chapter, self.minor = match.groups()
         self.urlbase = self.root + base
         ChapterExtractor.__init__(self, match, self.urlbase + "/1.html")
-        self.session.headers["Referer"] = self.root + "/"
 
     def metadata(self, page):
         manga, pos = text.extract(page, "<title>", "</title>")
@@ -71,36 +63,7 @@ class MangafoxMangaExtractor(MangaExtractor):
     root = "https://m.fanfox.net"
     chapterclass = MangafoxChapterExtractor
     pattern = BASE_PATTERN + r"(/manga/[^/?#]+)/?$"
-    test = (
-        ("https://fanfox.net/manga/kanojo_mo_kanojo", {
-            "pattern": MangafoxChapterExtractor.pattern,
-            "count": ">=60",
-            "keyword": {
-                "author": "HIROYUKI",
-                "chapter": int,
-                "chapter_minor": r"re:^(\.\d+)?$",
-                "chapter_string": r"re:(v\d+/)?c\d+",
-                "date": "type:datetime",
-                "description": "High school boy Naoya gets a confession from M"
-                               "omi, a cute and friendly girl. However, Naoya "
-                               "already has a girlfriend, Seki... but Momi is "
-                               "too good a catch to let go. Momi and Nagoya's "
-                               "goal becomes clear: convince Seki to accept be"
-                               "ing an item with the two of them. Will she bud"
-                               "ge?",
-                "lang": "en",
-                "language": "English",
-                "manga": "Kanojo mo Kanojo",
-                "tags": ["Comedy", "Romance", "School Life", "Shounen"],
-                "volume": int,
-            },
-        }),
-        ("https://mangafox.me/manga/shangri_la_frontier", {
-            "pattern": MangafoxChapterExtractor.pattern,
-            "count": ">=45",
-        }),
-        ("https://m.fanfox.net/manga/sentai_daishikkaku"),
-    )
+    example = "https://fanfox.net/manga/TITLE"
 
     def chapters(self, page):
         results = []
