@@ -210,7 +210,7 @@ def configure_logging(loglevel):
     root.setLevel(minlevel)
 
 
-def setup_logging_handler(key, fmt=LOG_FORMAT, lvl=LOG_LEVEL):
+def setup_logging_handler(key, fmt=LOG_FORMAT, lvl=LOG_LEVEL, mode="w"):
     """Setup a new logging handler"""
     opts = config.interpolate(("output",), key)
     if not opts:
@@ -219,11 +219,14 @@ def setup_logging_handler(key, fmt=LOG_FORMAT, lvl=LOG_LEVEL):
         opts = {"path": opts}
 
     path = opts.get("path")
-    mode = opts.get("mode", "w")
+    mode = opts.get("mode", mode)
     encoding = opts.get("encoding", "utf-8")
     try:
         path = util.expand_path(path)
         handler = logging.FileHandler(path, mode, encoding)
+    except FileNotFoundError:
+        os.makedirs(os.path.dirname(path))
+        handler = logging.FileHandler(path, mode, encoding)
     except (OSError, ValueError) as exc:
         logging.getLogger("gallery-dl").warning(
             "%s: %s", key, exc)
@@ -254,14 +257,14 @@ def stderr_write_flush(s):
     sys.stderr.flush()
 
 
-if sys.stdout.line_buffering:
+if getattr(sys.stdout, "line_buffering", None):
     def stdout_write(s):
         sys.stdout.write(s)
 else:
     stdout_write = stdout_write_flush
 
 
-if sys.stderr.line_buffering:
+if getattr(sys.stderr, "line_buffering", None):
     def stderr_write(s):
         sys.stderr.write(s)
 else:
