@@ -189,7 +189,7 @@ class MetadataPP(PostProcessor):
             tags = []
             extend = tags.extend
             for tagdict in taglists:
-                extend([x for x in tagdict.values() if x is not None])
+                extend([x for x in tagdict.values() if isinstance(x, str)])
             tags.sort()
 
         fp.write("\n".join(tags) + "\n")
@@ -206,7 +206,8 @@ class MetadataPP(PostProcessor):
             sort_keys=options.get("sort", False),
             separators=options.get("separators"),
             indent=options.get("indent", indent),
-            check_circular=False, default=str,
+            check_circular=False,
+            default=util.json_default,
         )
 
 
