@@ -9,14 +9,20 @@
 """Use metadata as file modification time"""
 
 from .common import PostProcessor
-from ..text import parse_int
+from .. import text, util, formatter
+from datetime import datetime
 
 
 class MtimePP(PostProcessor):
 
     def __init__(self, job, options):
         PostProcessor.__init__(self, job)
-        self.key = options.get("key", "date")
+        value = options.get("value")
+        if value:
+            self._get = formatter.parse(value, None, util.identity).format_map
+        else:
+            key = options.get("key", "date")
+            self._get = lambda kwdict: kwdict.get(key)
 
         events = options.get("event")
         if events is None:
@@ -26,9 +32,12 @@ class MtimePP(PostProcessor):
         job.register_hooks({event: self.run for event in events}, options)
 
     def run(self, pathfmt):
-        mtime = pathfmt.kwdict.get(self.key)
-        ts = getattr(mtime, "timestamp", None)
-        pathfmt.kwdict["_mtime"] = ts() if ts else parse_int(mtime)
+        mtime = self._get(pathfmt.kwdict)
+        pathfmt.kwdict["_mtime"] = (
+            util.datetime_to_timestamp(mtime)
+            if isinstance(mtime, datetime) else
+            text.parse_int(mtime)
+        )
 
 
 __postprocessor__ = MtimePP
