@@ -33,7 +33,8 @@ class MockDownloaderModule(Mock):
 class FakeJob():
 
     def __init__(self):
-        self.extractor = extractor.find("test:")
+        self.extractor = extractor.find("generic:https://example.org/")
+        self.extractor.initialize()
         self.pathfmt = path.PathFormat(self.extractor)
         self.out = output.NullOutput()
         self.get_logger = logging.getLogger
@@ -213,7 +214,8 @@ class TestHTTPDownloader(TestDownloaderBase):
         self.downloader.minsize = 100
         with self.assertLogs(self.downloader.log, "WARNING"):
             success = self.downloader.download(url, pathfmt)
-        self.assertFalse(success)
+        self.assertTrue(success)
+        self.assertEqual(pathfmt.temppath, "")
 
     def test_http_filesize_max(self):
         url = self.address + "/jpg"
@@ -221,7 +223,8 @@ class TestHTTPDownloader(TestDownloaderBase):
         self.downloader.maxsize = 100
         with self.assertLogs(self.downloader.log, "WARNING"):
             success = self.downloader.download(url, pathfmt)
-        self.assertFalse(success)
+        self.assertTrue(success)
+        self.assertEqual(pathfmt.temppath, "")
 
 
 class TestTextDownloader(TestDownloaderBase):
@@ -301,6 +304,7 @@ SAMPLES = {
     ("mp4" , b"????ftypavc1"),
     ("mp4" , b"????ftypiso3"),
     ("mp4" , b"????ftypM4V"),
+    ("mov" , b"????ftypqt  "),
     ("webm", b"\x1A\x45\xDF\xA3"),
     ("ogg" , b"OggS"),
     ("wav" , b"RIFF????WAVE"),
