@@ -24,6 +24,11 @@ from gallery_dl.extractor.directlink import DirectlinkExtractor  # noqa E402
 
 _list_classes = extractor._list_classes
 
+try:
+    from test import results
+except ImportError:
+    results = None
+
 
 class FakeExtractor(Extractor):
     category = "fake"
@@ -40,7 +45,7 @@ class TestExtractorModule(unittest.TestCase):
         "https://example.org/file.jpg",
         "tumblr:foobar",
         "oauth:flickr",
-        "test:pixiv:",
+        "generic:https://example.org/",
         "recursive:https://example.org/document.html",
     )
 
@@ -92,17 +97,29 @@ class TestExtractorModule(unittest.TestCase):
             with self.assertRaises(TypeError):
                 FakeExtractor.from_url(invalid)
 
-    def test_unique_pattern_matches(self):
-        try:
-            import test.results
-        except ImportError:
-            raise unittest.SkipTest("no test data")
+    @unittest.skipIf(not results, "no test data")
+    def test_categories(self):
+        for result in results.all():
+            url = result["#url"]
+            base, cat, sub = result["#category"]
+            try:
+                extr = result["#class"].from_url(url)
+            except ImportError as exc:
+                if exc.name in ("youtube_dl", "yt_dlp"):
+                    print("Skipping '{}' category checks".format(cat))
+                    continue
+                raise
+            self.assertEqual(extr.category, cat, url)
+            self.assertEqual(extr.subcategory, sub, url)
+            self.assertEqual(extr.basecategory, base, url)
 
+    @unittest.skipIf(not results, "no test data")
+    def test_unique_pattern_matches(self):
         # collect testcase URLs
         test_urls = []
         append = test_urls.append
 
-        for result in test.results.all():
+        for result in results.all():
             append((result["#url"], result["#class"]))
 
         # iterate over all testcase URLs
@@ -191,7 +208,7 @@ class TestExtractorModule(unittest.TestCase):
 class TestExtractorWait(unittest.TestCase):
 
     def test_wait_seconds(self):
-        extr = extractor.find("test:")
+        extr = extractor.find("generic:https://example.org/")
         seconds = 5
         until = time.time() + seconds
 
@@ -205,7 +222,7 @@ class TestExtractorWait(unittest.TestCase):
             self._assert_isotime(calls[0][1][1], until)
 
     def test_wait_until(self):
-        extr = extractor.find("test:")
+        extr = extractor.find("generic:https://example.org/")
         until = time.time() + 5
 
         with patch("time.sleep") as sleep, patch.object(extr, "log") as log:
@@ -220,7 +237,7 @@ class TestExtractorWait(unittest.TestCase):
             self._assert_isotime(calls[0][1][1], until)
 
     def test_wait_until_datetime(self):
-        extr = extractor.find("test:")
+        extr = extractor.find("generic:https://example.org/")
         until = datetime.utcnow() + timedelta(seconds=5)
         until_local = datetime.now() + timedelta(seconds=5)
 
