@@ -30,7 +30,8 @@ class MockPostprocessorModule(Mock):
 
 class FakeJob():
 
-    def __init__(self, extr=extractor.find("test:")):
+    def __init__(self, extr=extractor.find("generic:https://example.org/")):
+        extr.directory_fmt = ("{category}",)
         self.extractor = extr
         self.pathfmt = path.PathFormat(extr)
         self.out = output.NullOutput()
@@ -168,7 +169,7 @@ class ExecTest(BasePostprocessorTest):
 
     def test_command_string(self):
         self._create({
-            "command": "echo {} && rm {};",
+            "command": "echo {} {_path} {_directory} {_filename} && rm {};",
         })
 
         with patch("subprocess.Popen") as p:
@@ -178,7 +179,11 @@ class ExecTest(BasePostprocessorTest):
             self._trigger(("after",))
 
         p.assert_called_once_with(
-            "echo {0} && rm {0};".format(self.pathfmt.realpath), shell=True)
+            "echo {0} {0} {1} {2} && rm {0};".format(
+                self.pathfmt.realpath,
+                self.pathfmt.realdirectory,
+                self.pathfmt.filename),
+            shell=True)
         i.wait.assert_called_once_with()
 
     def test_command_list(self):
@@ -361,8 +366,8 @@ class MetadataTest(BasePostprocessorTest):
         self._create(
             {"mode": "tags"},
             {"tags": [
-                {"g": "foobar1", "m": "foobar2"},
-                {"g": None, "m": "foobarbaz"}
+                {"g": "foobar1", "m": "foobar2", "u": True},
+                {"g": None, "m": "foobarbaz", "u": [3, 4]},
             ]},
         )
         with patch("builtins.open", mock_open()) as m:
